import numpy as np


def wma(ohlcv, period=10, ohlcv_series="close"):
    
    """
    Return the weighted moving average (WMA) values
    """
    _ohlcv = ohlcv[[ohlcv_series]].copy(deep=True).sort_index(ascending=True)
    weights = np.arange(1, period + 1)
    indicator_values = _ohlcv[ohlcv_series].rolling(window=period, min_periods=period).\
        apply(lambda x: np.dot(x, weights)/weights.sum(), raw=True)

    return indicator_values


def dema(ohlcv, period=10, ohlcv_series="close"):
    """Return the double exponential moving average (DEMA) values

    Args:
        ohlcv ([type]): [description]
        period (int, optional): [description]. Defaults to 10.
        ohlcv_series (str, optional): [description]. Defaults to "close".

    Returns:
        [type]: [description]
    """
    _ohlcv = ohlcv[[ohlcv_series]].copy(deep=True)
    _ohlcv['ema'] = _ohlcv[ohlcv_series].ewm(span=period, min_periods=period).mean()
    _ohlcv['ema_ema'] = _ohlcv['ema'].ewm(span=period, min_periods=period).mean()
    indicator_values = 2 *_ohlcv['ema'] - _ohlcv['ema_ema']
    return indicator_values


def tema(ohlcv, period=10, ohlcv_series="close"):
    """
    Returns the triple exponential moving average (TEMA)
    """
    _ohlcv = ohlcv[[ohlcv_series]].copy(deep=True)
    _ohlcv['ema'] = _ohlcv[ohlcv_series].ewm(span=period, min_periods=period).mean()
    _ohlcv['ema_ema'] = _ohlcv['ema'].ewm(span=period, min_periods=period).mean()
    _ohlcv['ema_ema_ema'] = _ohlcv['ema_ema'].ewm(span=period, min_periods=period).mean()
    indicator_values = 3 *_ohlcv['ema'] - 3 * _ohlcv['ema_ema'] + _ohlcv['ema_ema_ema']
    return indicator_values



def trima(ohlcv, period=10, ohlcv_series="close"):
    """ Return the triangular moving average (TRIMA) values

    Args:
        ohlcv ([type]): [description]
        period (int, optional): [description]. Defaults to 10.
        ohlcv_series (str, optional): [description]. Defaults to "close".
    """
    pass


def kama():
    pass



def mama():
    pass


# Standard MACD function
def macd(ohlcv, short_period=12, long_period=26, signal_period=9, ohlcv_series="close"):
    """
    DESCRIPTION

    Gerald Appel developed Moving Average Convergence/Divergence as an indicator
    of the change in a security's underlying price trend. The theory suggests that
    when a price is trending, it is expected, from time to time, that speculative
    forces "test" the trend.  MACD shows characteristics of both a trending indicator
    and an oscillator. While the primary function is to identify turning points in a
    trend, the level at which the signals occur determines the strength of the reading.

    CALCULATION

    MACD Line:
     MACD=Period1 Exponential MA (Fast) -
          Period2 Exponential MA (Slow)
    MACD Signal Line:
     Sig=Period3 Exponential MA of MACD Line
    MACD Diff:
     Diff=MACD - Sig

    where:
    Period1 = N Period of the Fast Exponential MA
     The default Period1 is 12
    Period2 = N Period of the Slow Exponential MA
     The default Period2 is 26
    Period3 = N Period Exponential MA for the Signal Line
     The default Period3 is 9
    MACD Period:
     N Period = For daily, N=days; weekly, N=weeks, ...
    MACD omits non-trading days from computations.

    Formula:
    MACD Line: (12-day EMA - 26-day EMA)
    Signal Line: 9-day EMA of MACD Line
    MACD Histogram: MACD Line - Signal Line

    Validation: validated with results from Bloomberg
    """
    _ohlcv = ohlcv[[ohlcv_series]].copy(deep=True)
    _ohlcv["short"] = _ohlcv[ohlcv_series].ewm(span=short_period, min_periods=short_period).mean()
    _ohlcv["long"] = _ohlcv[ohlcv_series].ewm(span=long_period, min_periods=long_period).mean()
    _ohlcv["macd"] = _ohlcv["short"] - _ohlcv["long"]
    _ohlcv["signal"] = _ohlcv["macd"].ewm(span=signal_period, min_periods=signal_period).mean()
    _ohlcv["hist"] = _ohlcv["macd"] - _ohlcv["signal"]

    return _ohlcv[["macd", "signal", "hist"]]


import numpy as np


def rsi(ohlcv, period=14, ohlcv_series="close"):
    """
    RSI CALCULATION

    The very first calculations for average gain and average loss are simple n-period averages:

        First Average Gain = Sum of Gains over the past n periods / n.
        First Average Loss = Sum of Losses over the past n periods / n

    The second, and subsequent, calculations are based on the prior averages and the current gain loss:

        Average Gain = [(previous Average Gain) x (n-1) + current Gain] / n.
        Average Loss = [(previous Average Loss) x (n-1) + current Loss] / n.

    RS = Average Gain / Average Loss
    RSI = 100 - (100 / (1 + RS))

    """
    _ohlcv = ohlcv[[ohlcv_series]].copy(deep=True)
    _ohlcv["diff"] = _ohlcv[ohlcv_series].diff(periods=1)
    _ohlcv["diff_up"] = np.where(_ohlcv["diff"] >= 0, _ohlcv["diff"], 0)
    _ohlcv["diff_down"] = np.where(_ohlcv["diff"] < 0, _ohlcv["diff"], 0)
    # Calculate Average Gain and Average Loss
    _ohlcv[["rsi_u", "rsi_d"]] = _ohlcv[["diff_up", "diff_down"]].ewm(alpha=1 / period, min_periods=period).mean()
    _ohlcv["rs"] = abs(_ohlcv["rsi_u"]) / abs(_ohlcv["rsi_d"])
    indicator_values = 100 - (100 / (1 + _ohlcv["rs"]))

    return indicator_values


def ADD_RSI():
    pass
    


import pandas as pd


def load_daily(symbol="SPY", api_key='demo', start_date=None, end_date=None):
    try:
        file_url = f'https://www.alphavantage.co/query?function=TIME_SERIES_DAILY_ADJUSTED' \
                   f'&symbol={symbol}' \
                   f'&apikey={api_key}' \
                   f'&datatype=csv' \
                   f'&outputsize=full'

        _df = pd.read_csv(file_url,
                          parse_dates=['timestamp'],
                          infer_datetime_format=True,
                          index_col='timestamp')

        _df = _df[['open', 'high', 'low', 'close', 'volume']]

        if start_date:
            _df = _df[_df.index >= start_date]
        if end_date:
            _df = _df[_df.index <= end_date]

        _df = _df.sort_index(ascending=True)

        return _df
    except Exception as e:
        print(f'An error occurred. \nError Message is {e}')



# William %R

"""
WLPR allows you to plot the Williams %R value for a selected security.
It is useful in recognizing potential turning points to help make entry/exit
decisions and display event tracks. Use WLPR to determine overbought/oversold
levels: the Williams %R oscillator identifies whether a security is trading
at a relative high or low in relation to the highs and lows of a look back
period based on selected periodicity.  The default period for the indicator
is 14 periods for all chart types; intraday, daily, weekly, monthly.

WLPR Calculation

WLPR=-100*(N Period Highest High - Close) /
          (N Period Highest High - N Period Lowest Low)

where:
Highest High=The highest High over the specified N Period.
Lowest Low  =The lowest Low over the specified N Period.
N Period = For daily, N=days; weekly, N=weeks, ...
 The default is 14

WLPR omits non-trading days from computations.


"""

def wlpr(ohlcv, period=14, ohlcv_series="close"):
    """
    VALIDATION: validated with results from Bloomberg and TradingView
    """
    _ohlcv = ohlcv[['high', 'low', ohlcv_series]].copy(deep=True)
    _ohlcv['n_high'] = _ohlcv['high'].rolling(window=period, min_periods=period).max()
    _ohlcv['n_low'] = _ohlcv['low'].rolling(window=period, min_periods=period).min()

    indicator_values = -100 * (_ohlcv['n_high'] - _ohlcv[ohlcv_series]) / (_ohlcv['n_high'] - _ohlcv['n_low'])
    return indicator_values



# import pandas as pd


def sma(ohlcv, period=10, ohlcv_series="close"):
    """
    Calculate simple moving average of the input data

    :param _ohlcv: ohlcv dataframe
    :type _ohlcv: pd.DataFrame
    :param period: how many periods to look back
    :type period: int
    :param ohlcv_series: which of the o/h/l/v to use
    :type ohlcv_series: str
    :return: pd.series
    """
    _ohlcv = ohlcv[[ohlcv_series]].copy(deep=True)
    indicator_values = _ohlcv[ohlcv_series].rolling(window=period, min_periods=period).mean()

    return indicator_values


def sma_dif(_ohlcv, short_period=12, long_period=26, percent_diff=True, ohlcv_series="close"):
    """
    If the value is above 0,
    it means short line is above the long line,
    momentum is strong
    """
    short = _ohlcv[ohlcv_series].rolling(window=short_period, min_periods=short_period).mean()
    long = _ohlcv[ohlcv_series].rolling(window=long_period, min_periods=long_period).mean()
    indicator_col = short / long - 1 if percent_diff else short - long

    return indicator_col



# Bollinger Bands (BB)

"""
There are three bands when using Bollinger Bands

Middle Band – 20 Day Simple Moving Average
Upper Band – 20 Day Simple Moving Average + (Standard Deviation x 2)
Lower Band – 20 Day Simple Moving Average - (Standard Deviation x 2)
"""


def _bb_calc(ohlcv, period=20, number_of_std=2, ohlcv_series="close"):
    """RETURN Pandas DataFrame the contains all the necessary values/calculations of the Bollinger Band Indicator"""
    _ohlcv = ohlcv[[ohlcv_series]].copy(deep=True)
    _ohlcv["mid_band"] = _ohlcv[ohlcv_series].rolling(window=period, min_periods=period).mean()
    # ddof=0 means Degree of Freedom is 0. This calculates the population STD,
    # rather than the sample STD.
    _ohlcv["std"] = _ohlcv[ohlcv_series].rolling(window=period, min_periods=period).std(ddof=0)
    _ohlcv["lower_band"] = _ohlcv["mid_band"] - number_of_std * _ohlcv["std"]
    _ohlcv["upper_band"] = _ohlcv["mid_band"] + number_of_std * _ohlcv["std"]
    return _ohlcv


def boll(ohlcv, period=20, number_of_std=2, ohlcv_series="close"):
    """RETURN Three variables for the Bollinger Band indicator"""
    _ohlcv = _bb_calc(ohlcv, period, number_of_std, ohlcv_series)
    return _ohlcv[["lower_band", "mid_band", "upper_band"]]


def percent_b(ohlcv, period=20, number_of_std=2, ohlcv_series="close"):
    _ohlcv = _bb_calc(ohlcv, period, number_of_std, ohlcv_series)
    indicator_values = (_ohlcv[ohlcv_series] - _ohlcv['lower_band']) / (_ohlcv['upper_band'] - _ohlcv['lower_band'])
    return indicator_values





def ema(ohlcv, period=10, ohlcv_series="close"):
    """

    Validation: verified with numbers from TradingView
    """
    _ohlcv = ohlcv[[ohlcv_series]].copy(deep=True)
    indicator_values = _ohlcv[ohlcv_series].ewm(span=period, min_periods=period).mean()
    return indicator_values



# Rate of Change (RoC)

def roc(ohlcv, period=1, ohlcv_series="close"):
    """
    CALCULATION
    ROC = ( Current Price - Price N periods ago ) / Price N periods ago
    """
    _ohlcv = ohlcv[[ohlcv_series]].copy(deep=True)
    indicator_values = _ohlcv[ohlcv_series].pct_change(period)
    return indicator_values



"""
The Commodity Channel Index (CMCI), developed by Donald Lambert,
measures the variation of a security's price from its statistical mean.

The CMCI can be used to identify possible divergences that may indicate
a forthcoming trend for a selected security. A CMCI indicator falling
below a value of -100 indicates an oversold condition. A buy signal is
triggered when the indicator crosses -100 from below. Similarly, a CCI
value greater than 100 indicates an overbought condition. A sell signal
is triggered when the indicator crosses 100 from above.

CMCI Calculations
CMCI=(HLC3 - N Period Simple MA of HLC3) /
     (0.015 * N Period Mean Deviation of HLC3)

where:
HLC3 = (High + Low + Close) / 3
N Period= Number of data points to use;

"""
import numpy as np


def cmci(ohlcv, period=13):
    """
    VALIDATION

    Results are similar to Bloomberg data to the 4th digit after the decimal point.
    But very different from TradingView data.
    In some formulas, high and low are the highs and lows of the past N days.
    """
    _ohlcv = ohlcv[['high', 'low', 'close']].copy(deep=True)
    _ohlcv['hlc3'] = _ohlcv[['high', 'low', 'close']].mean(axis=1)
    _ohlcv['hlc3_ma'] = _ohlcv['hlc3'].rolling(window=period, min_periods=period).mean()
    _ohlcv['hlc3_mad'] = _ohlcv['hlc3'].rolling(window=period, min_periods=period).apply(
        lambda x: np.fabs(x - x.mean()).mean())

    indicator_values = (_ohlcv['hlc3'] - _ohlcv['hlc3_ma']) / (0.015 * _ohlcv['hlc3_mad'])
    return indicator_values



import pandas as pd
import numpy as np

def load_historical(symbol="SPY",
                    frequency='daily',
                    locale='us',
                    start_date=None,
                    end_date=None):

    try:
        file_url = f'https://data.qrtt.org/equity/{locale}/historical/{frequency}/{symbol}.csv'
        _df = pd.read_csv(file_url,
                          parse_dates=['timestamp'],
                          infer_datetime_format=True,
                          index_col='timestamp')
        # _df.index = _df.index.tz_localize(None)
        if start_date:
            _df = _df[_df.index >= start_date]
        if end_date:
            _df = _df[_df.index <= end_date]
        return _df
    except Exception as e:
        print(f'An error occurred. \nError Message is {e}')



from setuptools import setup, find_packages


setup(name='qrtt',
      version='0.0.1',
      description='Quantitative research and trading tools.',
      url='https://qrtt.org',
      download_url = 'https://github.com/leopoldsw/qrtt/archive/v0.01.tar.gz',
      author='Leopold W.',
      author_email='lsw@lwco.com',
      packages=find_packages(exclude=("tests", "tests_dev")),
      install_requires=['pandas', 'numpy', 'matplotlib', 'seaborn'],
      )
